# Löb's Theorem and Provability Predicates in Rocq

This archive contains the Rocq code accompanying [Janis Bailitis'](https://www.ps.uni-saarland.de/~bailitis/) [Bachelor's thesis](https://www.ps.uni-saarland.de/~bailitis/bachelor.php) at the [Programming Systems Lab](https://www.ps.uni-saarland.de) at [Saarland University](https://www.uni-saarland.de).

RocqDoc documentation can be found [here](https://www.ps.uni-saarland.de/~bailitis/bachelor/Coq/toc.html).

The mechanisation consists of the following files:
- `Definitions.v`: Some key definitions, including Gödel encodings
- `External_Provability.v`: Definition of external provability predicate, and proof that CT_Q implies existence of these predicates
- `Diagonal_Lemma.v`: Proof of the diagonal lemma and Carnap's semantic diagonalisation equivalence assuming CT_Q
- `Limitative_Theorems.v`: The indefinability theorem, Tarski's theorem, and some variants of Gödel's first incompleteness theorem assuming CT_Q
- `Loeb.v` Proof of Löb's theorem parameterised against a provability predicate satisfying the HBL conditions and a theory admitting the diagonal lemma
- `Multivariate_ctq.v`: Technical development generalising CT_Q to functions taking multiple arguments
- `hilbert_system/Hilbert_System.v`: Definition of a Hilbert system for first-order logic
- `hilbert_system/Hilbert_System_Deduction_Facts.v`: Proof that natural deduction and Hilbert system are equivalent
- `internal_provability/PA_lists_Signature.v`: Definitions of the signature of PA extended with native list and syntax functions
- `internal_provability/QEqLiFull.v`: Axioms of extended PA
- `internal_provability/Prov_definition.v`: Definition of an internal provability predicate for extended PA alongside technical routine verifications for this predicate
- `internal_provability/HBL.v`: Proof of the modus ponens rule and necessitation for the predicate defined in `Prov_definition.v`
- `internal_provability/Mostowski.v`: Proof that external provability predicates are not strong enough for Gödel's second incompleteness theorem
- The `util` directory contains some utility lemmas for vector manipulation and ND systems


## Installation Instructions

### Manual installation

The code has been tested on Coq `8.20`, compiled with OCaml `4.14.2`. It requires the [Rocq Library for First-Order Logic](https://github.com/uds-psl/coq-library-fol) for Coq `8.20`, available [here](https://github.com/uds-psl/coq-library-fol/tree/coq-8.20). This link also details the installation instructions for the library, but make sure to use OCaml version `4.14.2`, and not `4.14.0`, as suggested in the library's installation instructions. Once the dependencies are installed, preferably in a new opam switch, the mechanisation can be built as follows (make sure the environment is in sync with the opam switch):

- navigate to the root directory (the one that contains the file `_CoqProject`)
- `make all` builds the whole mechanisation and might take >30min (the bottleneck being  `internal_provability/Prov_definition.v`; you may wish to remove `internal_provability/HBL.v` and `internal_provability/Prov_definition.v` from `_CoqProject` to get significantly faster compilation times at the cost of not having a compiled mechanisation of the internal provability predicate)
- `make clean` removes all build files


### Rocq version

Be careful that this mechanisation has only been tested under Coq `8.20`. Future Coq/Rocq versions may work when the appropriate version of the [Rocq Library for First-Order Logic](https://github.com/uds-psl/coq-library-fol) is installed, but this is untested at the time being.

### Troubleshooting

In some cases, one needs to fiddle with the OCaml version. On the author's machine, the build succeeded with OCaml `4.14.2`. If that doesn't work, try OCaml `4.14.0`.


## Future Development

Parts of this mechanisation are currently being included into the [Rocq Library for First-Order Logic](https://github.com/uds-psl/coq-library-fol).